#ifndef VKTrace_H
#define VKTrace_H
/* *******************************************************************
*
* Copyright (c) 1989-1991 by CAD Language Systems, Inc.  All rights reserved.
*
* This file contains CLSI Confidential Information.
* Handle in accordance with contractual confidentiality requirements.
*
* HEADER FILE: VKTrace.h
*
* DESCRIPTION:
*	Instance registration and execution tracing functions of Symbolic
*	Information Interface.
*
* CONSTANTS:
*	<constants>	<constants defined in this header file >
*
* TYPES:
*	<types>		<types defined in this header file >
*
* GLOBAL VARIABLES:
*	<globals>	<global variables defined in this header file >
*
* ROUTINES:
*	...
*
* HISTORY:
*	20-FEB-90 DSB	Adding VK_InstantiateComponent
*	22-FEB-90 DSB	Moved InstantiateComponent from kernel to RTL
*	14-JUN-90 DSB	Created VKTrace.h
*	15-JUN-90 DSB	Added VK_tDebugCallType
*	18-DEC-90 DSB	Added kludge version of VK_TraceIfGenerate
*	18-DEC-90 DSB	Added kludge version of VK_TraceForGenerateGroup,
*			VK_TraceForGenerateBlock
*	26-DEC-90 DSB	Renamed and extended VK_tDebugCallType, 
*			Added initial VK_tUnitSymbInfo
*	02-JAN-90 DSB	Added real handling of generate statement regions
*	02-JAN-91 DSB	Changed from source file string to debug record
*	...
*	18-MAR-91 DSB	Defining call times; renamed VK_DebugXx to VK_TraceXx
*	18-MAR-91 DSB	Renamed VK_TraceXx to VK_TraceXx
*	20-MAR-91 DSB	Renamed VK_Create<Region> to VK_Trace<Region>; refining
*	21-MAR-91 DSB	Merged region instance registration with trace calls
*	26-MAR-91 DSB 	Folded region instance registration into one pair
*	29-MAR-91 DSB 	Added region information and decl./stmt. identification
*	02-APR-91 DSB	Using unit_info and exec_pos as position identification
*	10-MAY-91 JBR	W0041.  Removed null defines for VK_TraceCall,
*			VKTraceReturn, VK_TraceElab, and VK_TraceExec.
*	17-MAY-91 JBR	W0046.  Replaced VK_Create<Region> with macros.
*			Removed null #define for VK_TraceBegin.
*	17-MAY-91 DSB	Removed null #define for VK_TraceEnd; removed old
*			VKI's VK_tUnitSymbInfo (to private tUnitSymbInfo)
*
* SCCS ID: @(#)VKTrace.h	42.1 1/10/92
*
******************************************************************* */

#include "VKConfig.h"
#include "VKSymbTypes.h"	/* for VK_tSymbInfoHandle, etc. */
#include "VKCodeIntf.h"	/* for VK_tLARHeaderPtr, table names */


/* *******************************************************************

** Purposes:

Functions in VKTrace serve two main purposes.  The first is to supply 
identification information to the kernel.  The second is to give the kernel
control at certain points.

VK_TraceBegin informs the kernel about each instance of each static region, 
so the kernel can know the structure of the model hierarchy.  This applies to
design units and concurrent region statements (block statements, generate
statements, and configuration specifications for component instantiation
statements).

VK_TraceCall, VK_TraceReturn, VK_TraceBegin, and VK_TraceEnd inform the kernel
of (logical) calls and returns so the kernel can maintain a pointer to the 
region on the top of the (logical) call stack (to display the call stack).  
This applies to subprogram calls and nested static region elaboration.

All VK_TraceXx calls pass control back to the kernel for single stepping and
breakpoints.  This applies mainly to elaboration of declarations and 
concurrent statements and to execution of sequential statements (typical 
single stepping).  It also applies to (logical) calls and returns (for 
breaking on the calling of a subprogram or elaboration of a region, or for 
stepping to the return from a subprogram or the completion of elaboration of 
a region).

******************************************************************* */

/* *******************************************************************

* Protocol Elements:

** Each region data parameter is the local activation record pointer (LARP) of
	the current region.  For VK_TraceElab and VK_TraceExec, 
	encl_region_data is the LARP of the region enclosing the declaration or
	statement about	to be elaborated or executed.  For VK_TraceCall and 
	VK_TraceReturn, subprogram_data is the LARP of the called subprogram.
	For VK_TraceBegin and VK_TraceEnd, region_data is the LARP of the 
	concurrent region being elaborated. 

** The local activation record may contain information to identify the 
	region and the execution position.  (See VK_KERNEL_LAR_FIELDS and
	VK_KERNEL_IAR_FIELDS.)

** Each unit_info parameter identifies the design unit.  It provides access
	to the source file name and the execution tables for the design unit.

** Each exec_item_num parameter is an index in to the execution tables for
	the design unit.  It identifies the current execution position.

** Each region_info parameter (for VK_TraceBegin, VK_TraceEnd, VK_TraceCall, 
	and VK_TraceReturn) identifies the region.  It provides access to 
	symbolic information for the region, such as for displaying object
	values in the region.


** VK_TraceBegin and VK_TraceCall are to be called after the local activation 
	record for the region is created and after standard header fields of 
	the local activation record fields have been filled in.  Thus, the 
	kernel can fill in its special fields, and traverse to other regions.

** VK_TraceEnd and VK_TraceReturn are to be called before the local
	activation record is deallocated. 

** VK_TraceElab and VK_TraceExec are to be called immediately before the 
	generated C code for a VHDL declaration or statement executes.

** (Note: VK_TraceElab will be called for a static region before 
	VK_TraceBegin is called for that region.  VK_TraceBegin	is called for 
	a region before VK_TraceElab is called for any declaration or 
	statement contained in that region.)

** The standard header fields that are filled in before VK_TraceXx calls are:
	the interface activation record pointer field in the local activation 
	record, and the context region and caller region local activation 
	record pointer fields in the interface activation record.

** The fields likely defined and filled in by a kernel are:  a symbolic 
	information field identifying a region, an execution position field
	identifying the most-recently-started declaration or statement, and an 
	instance identification field for a static region.

******************************************************************* */

/* Note: The current code generator does not combine the local activation 
	records of a package declaration and an associated package body into 
	a record for the combined package.  */


/* *******************************************************************
*
* 1: Static region elaboration begin and end registration functions 
*
******************************************************************* */


/* *******************************************************************
*
* VK_TraceBegin - note the start of a static region's elaboration
*
* DESCRIPTION:
*	VK_TraceBegin is a debugging callback that is provided to inform the 
*	kernel of the start of elaboration of a static region.  It is called 
*	for the elaboration of each instance of a static region.  A static 
*	region is:  a package declaration, a package body, a design entity, a 
*	block statement, an explicit or equivalent process statement, or the 
*	equivalent block statement for an if or for generate statement, for a 
*	component instantiation statement, or for a configuration specifica-
*	tion or component specification.
*	Note: See common protocol notes above.
*
* HISTORY:
*	26-MAR-91 DSB 	Created (removed separate VK_Trace<Region> versions)
*	29-MAR-91 DSB 	Added region_info field
*	02-APR-91 DSB	Using unit_info and exec_pos as position identification
*
******************************************************************* */

void VK_TraceBegin(
#ifdef VK_ANSI_FN_PROTOS
	VK_tLARHeaderPtr region_data,	/* region instance data */
	VK_tSymbInfoHandle unit_info,	/* design unit identification */
	VK_tExecPosIndex exec_pos,	/* execution position identification */
	VK_tSymbInfoHandle region_info	/* region identification */
#endif
	);


/* *******************************************************************
*
* VK_TraceEnd - note the completion of a static region's elaboration
*
* DESCRIPTION:
*	VK_TraceEnd is a debugging callback that is provided to inform
*	the kernel of the end of elaboration of a static region.  It is 
*	called for the elaboration of each instance of a static region.
*	Note:  See common protocol notes above.
*
* HISTORY:
*	28-FEB-91 DSB	Reworking VKI
*	21-MAR-91 DSB	Split from VK_TraceReturn
*	29-MAR-91 DSB 	Added region_info field
*	02-APR-91 DSB	Using unit_info and exec_pos as position identification
*
******************************************************************* */

void VK_TraceEnd(
#ifdef VK_ANSI_FN_PROTOS
	VK_tLARHeaderPtr region_data,	/* region instance data */
	VK_tSymbInfoHandle unit_info,	/* design unit identification */
	VK_tExecPosIndex exec_pos,	/* execution position identification */
	VK_tSymbInfoHandle region_info	/* region symbolic information */
#endif
	);


/* *******************************************************************
*
* 2: Subprogram call and return trace functions
*
******************************************************************* */


/* *******************************************************************
*
* VK_TraceCall - note the call of a subprogram
*
* DESCRIPTION:
*	VK_TraceCall is a debugging callback that is provided to inform	the
*	kernel of the start of execution of a called subprogram.
*	Note:  See common protocol notes above.
*
* HISTORY:
*	28-FEB-91 DSB	Reworking VKI
*	29-MAR-91 DSB 	Added subprogram_info field
*	02-APR-91 DSB	Using unit_info and exec_pos as position identification
*
******************************************************************* */

void VK_TraceCall(
#ifdef VK_ANSI_FN_PROTOS
	VK_tLARHeaderPtr subprogram_data,
					/* subprogram activation data */
	VK_tSymbInfoHandle unit_info,	/* design unit identification */
	VK_tExecPosIndex exec_pos,	/* execution position identification */
	VK_tSymbInfoHandle subprogram_info
					/* subprogram symbolic information */
#endif
	);


/* *******************************************************************
*
* VK_TraceReturn - note return from a subprogram
*
* DESCRIPTION:
*	VK_TraceReturn is a debugging callback that is provided to inform the
*	kernel of the end of execution of a called subprogram.
*	Note:  See common protocol notes above.
*
* HISTORY:
*	28-FEB-91 DSB	Reworking VKI
*	29-MAR-91 DSB 	Added subprogram_info field
*	02-APR-91 DSB	Using unit_info and exec_pos as position identification
*
******************************************************************* */

void VK_TraceReturn(
#ifdef VK_ANSI_FN_PROTOS
	VK_tLARHeaderPtr subprogram_data,
					/* subprogram activation data */
	VK_tSymbInfoHandle unit_info,	/* design unit identification */
	VK_tExecPosIndex exec_pos,	/* execution position identification */
	VK_tSymbInfoHandle subprogram_info
					/* subprogram symbolic information */
#endif
	);


/* *******************************************************************
*
* 3: Elaboration and execution trace functions
*
******************************************************************* */


/* *******************************************************************
*
* VK_TraceElab - note the elaboration of a declaration or statement (etc.)
*
* DESCRIPTION:
*	VK_TraceElab is a debugging callback that is provided to inform the
*	kernel of the elaboration of a declaration or concurrent statement,
*	or creation of implicit signals or drivers.
*	Note:  See common protocol notes above.
*
* HISTORY:
*	28-FEB-91 DSB	Reworking VKI
*	29-MAR-91 DSB 	Added exec_item_num field
*	02-APR-91 DSB	Using unit_info and exec_pos as position identification
*
******************************************************************* */

void VK_TraceElab(
#ifdef VK_ANSI_FN_PROTOS
	VK_tLARHeaderPtr encl_region_data,
					/* enclosing region data */
	VK_tSymbInfoHandle unit_info,	/* design unit identification */
	VK_tExecPosIndex exec_pos	/* execution position identification */
#endif
	);


/* *******************************************************************
*
* VK_TraceExec - note the execution of a sequential statement
*
* DESCRIPTION:
*	VK_TraceExec is a debugging callback that is provided to inform the
*	kernel of the execution of a sequential statement.
*	Note:  See common protocol notes above.
*
* NOTES:
*   *	VK_TraceExec is to be called immediately before the execution code
*	for a VHDL sequential statement executes.
*   *   See common protocol notes above.
*
* HISTORY:
*	28-FEB-91 DSB	Reworking VKI
*	29-MAR-91 DSB 	Added exec_item_num field
*	02-APR-91 DSB	Using unit_info and exec_pos as position identification
*
******************************************************************* */

void VK_TraceExec(
#ifdef VK_ANSI_FN_PROTOS
	VK_tLARHeaderPtr encl_region_data,
					/* enclosing region data */
	VK_tSymbInfoHandle unit_info,	/* design unit identification */
	VK_tExecPosIndex exec_pos	/* execution position identification */
#endif
	);



#endif /* VKTrace_H */


