#ifndef CmdProcessor_H
#define CmdProcessor_H
/* *******************************************************************
*
* Copyright (c) 1989-1990 by CAD Language Systems, Inc.  All rights reserved.
*
* This file contains CLSI Confidential Information.
* Handle in accordance with contractual confidentiality requirements.
*
* HEADER FILE: CmdProcessor.h
*
* DESCRIPTION:  
*	Interface file for CmdProcessor.c.  
*
* CONSTANTS:
*
* TYPES:
*	tCmdTableId
*	tCmdName
*	tCmdProc
*	tTokenKind
*	tParseStatus
*	tCmdTable
*	tTokenBufferEntry
*	tTokenSeparator
*	tTokenBuffer
*	tTokenEntry
*	tToken
*
* GLOBAL VARIABLES:
*
* ROUTINES:
*   External:
*
*	ParseCmd - 		process contents of token_buf
*	ValidTableId - 		tests a user table id for validity
*	RegCmdTable - 		register a user command table
*	NextToken - 		get the next token from token_buffer
*	TokenValue - 		copy the value of token to buffer
*	SetTokenBuffer - 	initialize a token buffer (MACRO)
*	TokenType - 		return the type of a token (MACRO)
*	TokenBufferValue -	copy the value of token buffer to buffer
*	SetTokenSeparator -	initialize a token delimiter object
*	Pushback -		return a token to the token buffer
*
*   Internal:
*	IsPrefix -		determine if string1 is a prefix of string2
*	BuildSet -		build internal representation of delimiter set
*	InSet -			determine if an element is member of delimiter set
*	CommandType -		determine if a token is a command type
*
* HISTORY:
*       09-JAN-91 RM	Created
*	09-FEB-91 RM	Added user specified token delimiters.
*	04-APR-91 WAG	W0023.  Modified to conform to coding style standards.
*	02-MAY-91 DSB	Cleaning up generally
*
* SCCS ID: @(#)CmdProcessor.h	42.1 1/10/92
*
******************************************************************* */

#include "VKConfig.h"
#include "VKBasicTypes.h"


typedef int tCmdTableId;

typedef char *tCmdName;

typedef VK_tBoolean (*tCmdProc) ();

typedef enum {
        TK_NotKnown,		/* token not recognizable */
	TK_NotComplete,		/* found end of input while scanning token */
        TK_NoToken,		/* no more tokens in input */
	TK_Identifier,		/* [_a-zA-Z][_a-zA-Z0-9]* */
        TK_UserSpecified,	/* token delimited by user specified delimiters */
	TK_Number,		/* [0-9]+ */
        TK_Control,
	TK_Exclamation,
	TK_DQuote,
	TK_Pound,
	TK_Dollar,
	TK_Percent,
	TK_Ampersand,
	TK_SQuote,
	TK_LParenthesis,
	TK_RParenthesis,
	TK_Asterisk,
	TK_Plus,
	TK_Comma,
	TK_Dash,
	TK_Period,
	TK_FSlash,
	TK_Colon,
	TK_SemiColon,
	TK_LAngle,
	TK_Equal,
	TK_RAngle,
	TK_Question,
	TK_Bar,
	TK_At,
	TK_LSquare,
	TK_BSlash,
	TK_RSquare,
	TK_UpArrow,
	TK_Underline,
	TK_Tick,
	TK_LCurly,
	TK_RCurly,
	TK_Tilde
    } tTokenKind;

typedef enum {
	PS_Error,		/* internal error */
        PS_NotCmdTable,		/* invalid cmd table id */
	PS_CmdComplete,		/* cmd ok, routine returned True */
	PS_CmdIncomplete,	/* cmd ok, routine returned False */
	PS_NotCmd,		/* cmd is unrecognizable */
	PS_AmbiguosCmd,		/* cmd is ambiguous */
	PS_NoNext		/* no next token */
    } tParseStatus;

typedef struct sCmdTable {
	tCmdName cmd;
	tCmdProc proc;
    } tCmdTable[];

typedef struct sTokenBufferEntry {
	char *buffptr;
	int pos;
    } tTokenBufferEntry;

typedef tTokenBufferEntry tTokenBuffer[1];

typedef struct sTokenEntry {
        tTokenKind type;
	char *bptr;
	int start;
	int end;
    } tTokenEntry;

typedef tTokenEntry tToken[1];

#define TSSize 132                 /* 0-127 + 4 */
typedef char tTokenSeparator[TSSize];


#define TS_WhiteSpace	0x80
#define TS_HT		0x09
#define TS_NL		0x0A
#define TS_CR		0x0D
#define TS_SP	 	0x20 
#define TS_Exclamation	0x21
#define TS_DQuote	0x22
#define TS_Pound	0x23
#define TS_Dollar	0x24
#define TS_Percent	0x25
#define TS_Ampersand	0x26
#define TS_SQuote	0x27
#define TS_LParenthesis	0x28
#define TS_RParenthesis	0x29
#define TS_Asterisk	0x2A
#define TS_Plus		0x2B
#define TS_Comma	0x2C
#define TS_Dash		0x2D
#define TS_Period	0x2E
#define TS_FSlash	0x2F
#define TS_Colon	0x3A
#define TS_SemiColon	0x3B
#define TS_LAngle	0x3C
#define TS_Equal	0x3D
#define TS_RAngle	0x3E
#define TS_Question	0x3F
#define TS_At		0x40
#define TS_LSquare	0x5B
#define TS_BSlash	0x5C
#define TS_RSquare	0x5D
#define TS_UpArrow	0x5E
#define TS_Underline	0x5F
#define TS_Tick		0x60
#define TS_LCurly	0x7B
#define TS_Bar		0x7C  
#define TS_RCurly	0x7D
#define TS_Tilde	0x7E


tCmdTableId eRegCmdTable();

#define RegCmdTable(table, size) \
       eRegCmdTable(table, size)


tParseStatus eParse();

#define ParseCmd(token_buf, tid) \
       eParseCmd(token_buf, tid)


void eNextToken();

#define NextToken(token_buf, token, token_separator) \
       eNextToken(token_buf, token, token_separator)


void eSetTokenSeparator();
#define SetTokenSeparator eSetTokenSeparator      /* var. arg. list */


void eSetTokenBuffer();

#define SetTokenBuffer(token_buf, buffer) \
        token_buf[0].buffptr = buffer; \
        token_buf[0].pos = 0


#define TokenType(token)	((token[0]).type)


void eTokenValue();

#define TokenValue(token, buffer, size) \
       eTokenValue(token, buffer, size)


void eTokenBufferValue();

#define TokenBufferValue(tok_buf, buffer, size) \
       eTokenBufferValue(tok_buf, buffer, size)


void ePushback();

#define Pushback(tok_buf, token) \
       ePushback(tok_buf, token)


VK_tBoolean eValidTableId();

#define ValidTableId(tid) \
       eValidTableId(tid)




tParseStatus eParseCmd(
#ifdef ANSI_FN_PROTOS
	tTokenBuffer token_buf,
	tCmdTableId tid
#endif
	);

#endif /* CmdProcessor_H */

