#ifndef VKSymbTypes_H
#define VKSymbTypes_H
/* *******************************************************************
*
* Copyright (c) 1989-1991 by CAD Language Systems, Inc.  All rights reserved.
*
* This file contains CLSI Confidential Information.
* Handle in accordance with contractual confidentiality requirements.
*
* HEADER FILE: VKSymbTypes.h
*
* DESCRIPTION:
*	This module contains most of the types for the Symbolic Information
*	Interface of the VKI.
*
* CONSTANTS:
*	...
*
* TYPES:
*	...
*
* GLOBAL VARIABLES:
*	none
*
* ROUTINES:
*	...
*
* HISTORY:
*	15-MAR-91 DSB 	Created; split out from VKRegister.h
*	15-MAR-91 DSB	Moved VK_tExecPosIndex to VKSymbTypes.h
*	15-MAR-91 DSB	Moved VK_tSymbInfoHandle to VKSymbTypes.h
*	01-APR-91 DSB	Reworking execution identification and control
*	02-APR-91 DSB	Added VK_tExecPosKind; reworking execution tables
*	03-APR-91 DSB	More execution tables rework, documentation, & cleanup
*	09-APR-91 DSB	Adding explicit breakpoint table, types
*	12-APR-91 DSB	Added VK_XxxArrayAddr types
*	22-APR-91 DSB	Added VK_tTimeStampPtr
*	14-MAY-91 JBR	W0046.  Created Test Kernel version of
*			VK_KERNEL_LAR_FIELDS AND RELATED MACROS.
*	21-MAY-91 DSB	Cleaning: removed old VK_Exec, VK_Elab, etc.
*
* SCCS ID: @(#)VKSymbTypes.h	42.1 1/10/92
*
******************************************************************* */

#include "VKConfig.h"
#include "VKBasicTypes.h"	/* for VK_tIntegerData, VK_tLargeData */


/* *******************************************************************
*
* 1: execution position identification and execution control types
*
******************************************************************* */

/* *******************************************************************
*
* Notes:
*   *   An "execution position" is a point in the generated code at which
*	there is a VK_TraceXx call (for breakpoints and stepping in debugging,
*	and for maintaining the current execution position).  There is an 
*	execution position for each VHDL declaration or statement, at the 
*	beginning and end of each VHDL region, and at several other places.
*   *	Each unit has an execution position identification (trace) table.  The
*	table is an array that contains an entry for each execution position
*	in the unit.  Each entry identifies one execution position, with the
*	source file line number and the position kind (e.g., subprogram start 
*	vs. statement execution).
*   *	Each unit also has an execution control (breakpoint) table.  It also 
*	is an array with an entry for each execution position in the unit.
*	Each entry contains control information (e.g., a breakpoint bit) for 
*	one execution position.  The kernel defines the element type of the
*	table.
*   *   The generated code initializes the execution position identification 
*	table statically.
*   *   The kernel may initialize the execution control table when it is
*	registered.
*   *	An execution position index identifies a specific execution position
*	(its VK_TraceXx call and its table entries).
*
******************************************************************* */

typedef int VK_tExecPosIndex;		/* trace/breakpoint table index for 
					   execution position */

typedef int VK_tLineNumber;		/* source file line number */


/* *******************************************************************
*
* type VK_tExecPosKind - execution position's elaboration/execution kind
*
* USAGE:  Type name and structure defined by VKI; enumerators used in
*	generated code; type name used for VKI function parameters.
* 
******************************************************************* */
typedef enum {
	VK_BadExecPosKind,	/* -special for dummy position entry */
	VK_Begin,		/* -beginning of static region elaboration */
	VK_End,			/* -end of static region elaboration */
	VK_Call,		/* -subprogram execution start */
	VK_Return,		/* -subprogram execution end */
	VK_ElabDecl,		/* -elaboration of declaration */
	VK_ElabStmt,		/* -elaboration of concurrent statement */
	VK_ElabSpec,		/* -elaboration of driver or impl. signal */
	VK_ElabConfig,		/* -elaboration of configuration item */
	VK_ExecStmt		/* -execution of sequential statement */
    } VK_tExecPosKind;		/* execution position kind */
/* Specification:  Integer values do not matter. */

/* Note: VK_Begin and VK_TraceBegin do NOT correspond to the position of the
	"begin" keyword */


/* *******************************************************************
*
* type VK_tTraceTableEntry - execution position ident. (trace) table entry
*
* DESCRIPTION:
*	Type VK_tTraceTableEntry represents the information stored to 
*	identify one execution position.  Field line_number is the source
*	file line number.  Field exec_pos_type is the kind of execution
*	position.
*
* USAGE:  Used by generated code to declare and initialize table.
*
******************************************************************* */

typedef struct {
	VK_tLineNumber line_number;	/* -source line number */
	VK_tExecPosKind exec_pos_kind;	/* -execution position kind */
    } VK_tTraceTableEntry;		/* execution position information */



/* *******************************************************************
*
* type VK_tTraceTable - execution position ident. (trace) table for unit
*
* DESCRIPTION:
*	Type VK_tTraceTable represents the table of all execution position
*	information for a unit.  It is indexed by an execution position
*	index.
*
* USAGE:  VKI defines it; VKI functions use it; generated code uses it.
*
******************************************************************* */

typedef VK_tTraceTableEntry VK_tTraceTable[];
					/* information for positions in unit */


/* *******************************************************************
*
* type VK_tBreakTableEntry - execution control (breakpoint) table entry
*
* DESCRIPTION:
*	Type VK_tBreakTableEntry represents the control information for one
*	execution position.  Fields are defined by the kernel.
*
* USAGE:  VKI defines type name; kernel defines types definition; generated 
*	code uses type name to declare static array variable, but does not 
*	know fields; kernel likely uses definition in VK_TraceXx 
*	implemented with macros.
*
******************************************************************* */

#if ! TK_SPECIFIC
  typedef int VK_tBreakTableEntry;	/* (whatever kernel wants) */
#else
  typedef VK_tBoolean VK_tBreakTableEntry; /* test kernel has break bit */
#endif

/* Specification: VK_tBreakTableEntry may be any type (scalar or composite). */


/* *******************************************************************
*
* type VK_tBreakTable - execution control (break) table for unit
*
* DESCRIPTION:
*	Type VK_tBreakTable represents the table of all execution control
*	information for a unit.  It is indexed by an execution position
*	index.
*
* USAGE:  VKI defines it; VKI functions use it; generated code uses it.
*
******************************************************************* */

typedef VK_tBreakTableEntry VK_tBreakTable[];
					/* information for positions in unit */


/* *******************************************************************
*
* macro VK_KERNEL_UNIT_DECLS - declare unit variables for kernel
*
* DECLARATION:  VK_KERNEL_UNIT_DECLS is a special syntactic macro.  It takes
*	no parameters.
*
* DESCRIPTION:
*	VK_KERNEL_UNIT_DECLS is a hook for the kernel to define things to
*	declare in each design unit's generated module.  The generated code
*	refers to VK_KERNEL_UNIT_DECLS (before any VK_TraceXx calls and 
*	before the unit registration call).
*
* USAGE: VK_KERNEL_UNIT_DECLS is defined by the kernel; it is used by
*	the VKI and the generated code.
*
* NOTES:
*   *   The generated code does not terminate the macro call with a semicolon.
*   *   The prefixes "K_" and "k_" are reserved to the kernel for any global
*	names it defines (e.g., for (static) global variable names).
*
******************************************************************* */

#define VK_KERNEL_UNIT_DECLS() 		/* (whatever the kernel wants) */


/* *******************************************************************
*
* macro VK_KERNEL_IAR_FIELDS - declare IAR fields for kernel
* macro VK_KERNEL_LAR_FIELDS - declare LAR fields for kernel
*
* DECLARATION:  VK_KERNEL_IAR_FIELDS and VK_KERNEL_LAR_FIELDS are special 
*	syntactic macros.  They take no parameters.
*
* DESCRIPTION:
*	VK_KERNEL_IAR_FIELDS and VK_KERNEL_LAR_FIELDS are hooks for the kernel
*	to define whatever additional fields it wants in generated interface
*	and local activation records (IARs and LARs), e.g., for region 
*	identification and execution position information.  The generated 
*	code uses VK_KERNEL_IAR_FIELDS in the declaration of any IAR (before 
*	any other fields), and uses VK_KERNEL_LAR_FIELDS in the declaration 
*	of any LAR (before any other fields).
*
* USAGE:  The interface to these macros is defined by the VKI and used by the
*	generated code.  The actual contents are defined by the kernel. 
*
* NOTES:
*   *   The prefixes "K_" and "k_" are reserved to the kernel for any field
*	names it defines.
*
******************************************************************* */

#define VK_KERNEL_LAR_FIELDS()		/* nothing, or whatever kernel wants */
#define VK_KERNEL_IAR_FIELDS()		/* nothing, or whatever kernel wants */

#ifdef TK_SPECIFIC			/* TEST KERNEL IMPLEMENTATION */

#undef VK_KERNEL_LAR_FIELDS		/* cancel template definition */

/* Notes: Usually, hold on to LARP as handle to region data; identify region 
	using LAR field. */

/* *******************************************************************
*
* TK_tRegionHandle - for TEST KERNEL IMPLEMENTATION ONLY
*
******************************************************************* */
typedef struct TK_sRegion * TK_tRegionHandle;	/* opaque region handle */


/* *******************************************************************
*
* macro VK_KERNEL_LAR_FIELDS - for TEST KERNEL IMPLEMENTATION ONLY
*
* HISTORY:
*	17-MAY-91 DSB	Moved field macros to private test kernel file
*	17-MAY-91 DSB	Added k_region_info, k_exec_pos_index fields
*
******************************************************************* */
#define VK_KERNEL_LAR_FIELDS() \
	TK_tRegionHandle k_region_handle; /* kernel's info for region */ \
	VK_tSymbInfoHandle k_region_info; \
					/* region info. to identify region */ \
	VK_tSymbInfoHandle k_unit_info;	/* unit info. for exec. position */ \
	VK_tExecPosIndex k_exec_pos_index; \
					/* index for exec. position */ 
					

/* Hints, Ideas:
    *	The k_region_info field is one way for a kernel keep track of
	instances of (static) regions.
    *	The k_unit_info and k_exec_pos_index field pair is one way for the 
	kernel to track the execution position (to get the line number of the 
	last declaration, statement, etc.). 
    *   A trace table entry address could replace the unit_info and 
	exec_pos_index fields (and the region_info field).  Unit and region
	information would be looked up that same way that a native-mode de-
	bugger scans debug tables for relocated program counter addresses.
*/
#endif /* TK_SPECIFIC */


/* *******************************************************************
*
* 2: types for symbolic information
*
******************************************************************* */

/* *******************************************************************
*
* basic types
*
******************************************************************* */

typedef VK_tCString VK_tName;		/* VHDL simple name of item, etc. */

typedef int VK_tEnclItemNumber;		/* item number in enclosing region */


/* *******************************************************************
*
* types VK_tXxArray, VK_tXxArrayAddr - enumeration/physical type table types
*
* DESCRIPTION:
*	Array types are for generated code to declare static variables with
*	initialization.  Address types are for VKI function parameters.
*
******************************************************************* */


typedef VK_tName VK_tStringArray[];	/* string pointers array object */
typedef VK_tName * VK_tStringArrayAddr;	/* string pointers array address */

typedef VK_tIntegerData VK_tIntegerArray[];
					/* integer values array object */
typedef VK_tIntegerData * VK_tIntegerArrayAddr;
					/* integer values array address */

typedef VK_tLargeData VK_tLargeArray[];	/* large values array object */
typedef VK_tLargeData * VK_tLargeArrayAddr;
					/* large values array address */


/* *******************************************************************
*
* types for object information
*
******************************************************************* */

typedef VK_tByteSize VK_tFieldOffset;	/* byte offset of field in interface or
					   local activation record structure */

typedef enum { 
	VK_BadSignalKind,	        /* -(not required by VKI) */
	VK_UnguardedKind,		/* -unguarded signal */
	VK_BusKind, 			/* -bus signal */
	VK_RegisterKind 		/* -register signal */
    } VK_tSignalKind;			/* VHDL signal kind */
/* Specification:  Integer values do not matter.  Values shown are test 
	kernel's choice. */

typedef enum { 
	VK_BadMode,		        /* -(not required by VKI) */
	VK_InMode,			/* -interface mode in */
	VK_InoutMode,			/* -interface mode inout */
	VK_OutMode,			/* -interface mode out */
	VK_BufferMode,			/* -interface mode buffer */
	VK_LinkageMode,			/* -interface mode linkage */
	VK_InternalMode			/* -(not required by VKI) */
    } VK_tMode;				/* VHDL interface or file mode */
/* Specification:  Integer values do not matter.  Values shown are test 
	kernel's choice. */


/* *******************************************************************
*
* type VK_tTimeStamp/-Ptr - architecture analysis time stamp (for config.)
*
******************************************************************* */

typedef struct {
	short int year;
	char month;
	char day;
	char hour;
	char minute;
	char second;
	char hundredth;
    } VK_tTimeStamp;			/* time stamp */

typedef VK_tTimeStamp * VK_tTimeStampPtr;
					/* time stamp pointer */


/* *******************************************************************
*
* VK_tSymbInfoHandle - handle to registered symbolic information for
*	design unit, declaration, statement, or other item
*
******************************************************************* */

typedef struct VK_sSymbInfo * VK_tSymbInfoHandle;


#endif /* VKSymbTypes_H */
